package cn.com.qmth.stmms.ms.marking.api;

import cn.com.qmth.stmms.ms.commons.utils.SqlUtil;
import cn.com.qmth.stmms.ms.core.domain.ExamQuestion;
import cn.com.qmth.stmms.ms.core.domain.Level;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.user.MarkRight;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.assembler.LevelStatAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.MarkerAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.QuestionStatAssembler;
import cn.com.qmth.stmms.ms.marking.dto.LevelStatDTO;
import cn.com.qmth.stmms.ms.marking.dto.MarkerDTO;
import cn.com.qmth.stmms.ms.marking.dto.QuestionStatDTO;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 评卷中，评卷员的相关数据api
 * Created by zhengmin on 2016/10/14.
 */
@RestController
@RequestMapping("api/markers")
public class MakrerApi {
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(MakrerApi.class);

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private MarkTaskRepo markTaskRepo;

    @Autowired
    private MarkerAssembler markerAssembler;

    @Autowired
    private QuestionStatAssembler questionStatAssembler;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private LevelStatAssembler levelStatAssembler;

    @Autowired
    ExamQuestionRepo examQuestionRepo;

    @Autowired
    SqlUtil sqlUtil;

    /**
     * 评卷员信息
     *
     * @param markerId 评卷员用户id
     * @return
     */
    @RequestMapping(value = "{markerId}", method = RequestMethod.GET)
    public MarkerDTO getMarker(@PathVariable Long markerId) {
        MarkUser marker = markUserRepo.findOne(markerId);
        return markerAssembler.toDTO(marker);
    }

    /**
     * 评卷员列表
     *
     * @param workId  评卷工作id
     * @param subject 科目
     * @return
     */
    @RequestMapping(method = RequestMethod.GET)
    public List<MarkUser> list(@RequestParam Long workId,
                               @RequestParam Subject subject) {
        List<MarkUser> markUsers = markUserRepo.findByWorkIdAndSubjectAndRole(workId, subject, Role.MARKER);
        String markSubjectId = workId + "-" + subject.toString();
        MarkSubject markSubject = markSubjectRepo.findOne(markSubjectId);
        if (MarkStage.INIT == markSubject.getStage()) {
            markUsers = markUsers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_LEVELING || m.getMarkRight() == MarkRight.ALLOW_ALL).collect(Collectors.toList());
        } else if (MarkStage.LEVEL == markSubject.getStage()) {
            markUsers = markUsers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_SCORING || m.getMarkRight() == MarkRight.ALLOW_ALL).collect(Collectors.toList());
        }
        return markUsers;
    }

    /**
     * 评卷员分档数量及占比数据
     *
     * @param marker     评卷员用户id
     * @param questionId 试题id
     * @return
     */
    @RequestMapping(value = "{marker}/stat/levels", method = RequestMethod.GET)
    public List<LevelStatDTO> levelStats(@PathVariable MarkUser marker, @RequestParam Long questionId) throws Exception {
        List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
//        markTaskRepo.countGroupByLevel(marker.getId())
//                .forEach(o -> {
//                    levelStatAssembler.toDTO(o);
//                    levelStatDTOs.add();
//                });
        ExamQuestion examQuestion = examQuestionRepo.findOne(questionId);
        if (Objects.isNull(examQuestion)) {
            throw new Exception("当前考试试卷为空");
        }
        //获取当前考试当前科目所在地区的所有questionId
        List<ExamQuestion> examQuestionList = examQuestionRepo.findAllByAreaCodeAndSubject(examQuestion.getAreaCode(), examQuestion.getSubject());
        List<Long> questionIds = examQuestionList.stream()
                .map(m -> m.getId())
                .collect(Collectors.toList());
        StringBuilder startSql = new StringBuilder("select t.`result`,sum(if(t.questionId <> t.currentQuesId, 0, t.sumCount)) as sumCount from(SELECT t.result,COUNT(*) as sumCount,t.question_id as questionId,? as currentQuesId FROM mark_task t WHERE t.`question_id` in ( ");
        String endSql = " ) and t.stage = 1 GROUP BY t.`result`,t.question_id ORDER BY t.result) t group by t.`result` ";
        questionIds.forEach(o -> startSql.append(o).append(","));
        //当前老师当前试卷的评档次数
        StringBuilder sql = new StringBuilder(startSql.toString().substring(0, startSql.toString().length() - 1));
        sql.append(endSql);
        String finalSql = sqlUtil.sqlPrint(sql.toString(), questionId);
        List list = sqlUtil.execSqlForMap(finalSql);
        MarkSubject markSubject = markSubjectRepo.findOne(marker.getWorkId() + "-" + marker.getSubject().toString());
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        Map<String, Level> levelMap = levels.stream().collect(Collectors.toMap(Level::getCode, l -> l));
//        Map<String, LevelStatDTO> filterLevelMap = new HashMap<>();
        for (int i = 0; i < list.size(); i++) {
            Map map = (Map) list.get(i);
            List finalList = (List) map.values().stream()
                    .collect(Collectors.toList());
            Object[] o = finalList.toArray();
            LevelStatDTO levelStatDTO = levelStatAssembler.toDTO(o);
            if (levelStatDTO.getId() == null) {
                //当前老师未评档的次数
//              int gcount = markTaskRepo.countByLevel(marker.getId());
                int gcount = markTaskRepo.countByQuestionIdInAndStageAndResultIsNull(questionIds, MarkStage.LEVEL);
                levelStatDTO.setGcount(gcount);
                levelStatDTOs.add(levelStatDTO);
            } else {
                //当前老师每个档位已评档位的次数
//              int gcount = markTaskRepo.countByLevelAndResult(marker.getId(), levelStatDTO.getId().toString());
                int gcount = markTaskRepo.countByQuestionIdInAndStageAndResult(questionIds, MarkStage.LEVEL, levelStatDTO.getId().toString());
                levelStatDTO.setGcount(gcount);
            }
            if (Objects.nonNull(levelMap.get(levelStatDTO.getId()))) {
                levelStatDTOs.add(levelStatDTO);
            }
        }
        for (Level level : levels) {
            //所有档位依次比较
            long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(level.getCode())).count();
            if (count == 0) {
                LevelStatDTO dto = new LevelStatDTO();
                dto.setId(level.getCode());
                dto.setCount(0);
                dto.setPercent(0.0);

                //当前老师当前档位评档次数(所有考试)
//                int gcount = markTaskRepo.countByLevelAndResult(marker.getId(), level.getCode());
                int gcount = markTaskRepo.countByQuestionIdInAndStageAndResult(questionIds, MarkStage.LEVEL, level.getCode());
                dto.setGcount(gcount);
                levelStatDTOs.add(dto);
            }
        }

        //当前老师当前试卷的评档次数(不分档位)
//        long kdtotal = markTaskRepo.countByMarkerIdAndQuestionId(marker.getId(), questionId);
        long kdtotal = markTaskRepo.countByQuestionIdIn(questionIds);
        //当前老师所有的评档次数(不分档位)
//        long total = markTaskRepo.countByMarkerId(marker.getId());
        long total = markTaskRepo.countByQuestionId(questionId);

        levelStatDTOs.forEach(o -> {
            if (o.getId() != null) {
                double p = (double) o.getCount() / total;
                BigDecimal bd = new BigDecimal(p).setScale(3, RoundingMode.HALF_EVEN);
                o.setPercent(bd.doubleValue());

                double gp = (double) o.getGcount() / kdtotal;
                BigDecimal gbd = new BigDecimal(gp).setScale(3, RoundingMode.HALF_EVEN);
                o.setGpercent(gbd.doubleValue());
            }
        });
        return levelStatDTOs;
    }

    /**
     * 各试题统计数据
     *
     * @param marker 评卷员用户id
     * @return
     */
    @RequestMapping(value = "{marker}/stat/questions", method = RequestMethod.GET)
    public List<QuestionStatDTO> questions(@PathVariable MarkUser marker) {
        List<QuestionStatDTO> questionStatDTOs = new ArrayList<>();
        List<Object[]> qStats = null;
        MarkSubject markSubject = markSubjectRepo.findOne(marker.getWorkId() + "-" + marker.getSubject().toString());
        qStats = markTaskRepo.countGroupByQuestion(marker.getId(), markSubject.getStage().ordinal());
        if (qStats != null) {
            for (Object[] objects : qStats) {
                QuestionStatDTO dto = questionStatAssembler.toDTO(objects);
                questionStatDTOs.add(dto);
            }
        }
        return questionStatDTOs;
    }

}
